# -*- coding: UTF-8 -*-
from . import api_base
try:
    from StringIO import StringIO
except:
    from io import StringIO
import pandas as pd
import sys
from datetime import datetime
from .api_base import get_cache_key, get_data_from_cache, put_data_in_cache, pretty_traceback
import inspect
try:
    unicode
except:
    unicode = str

__doc__="南华"
def __NHIndexTickHistOneDayGet(ticker, tradeDate, field = "", pandas = "1"):
    """
    高频数据，获取一只南华指数在历史上某个交易日的Tick数据，指数范围仅包含以下6个：南华工业品指数、南华能化指数、南华贵金属指数、南华商品指数、南华金属指数、南华农产品指数
    
    :param ticker: NHCI-南华工业品指数、NHCI-南华能化指数、NHCI-南华贵金属指数、NHCI-南华商品指数、NHCI-南华金属指数、NHCI-南华农产品指数,可以是列表
    :param tradeDate: 清算日期，无夜盘的情况下同交易日期，格式为yyyymmdd
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/idxMarket/getNHIndexTickHistOneDay.csv?ispandas=1&') 
    requestString.append("ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    if not isinstance(tradeDate, str) and not isinstance(tradeDate, unicode):
        tradeDate = str(tradeDate)

    requestString.append("&tradeDate=%s"%(tradeDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__NHIndexTickHistOneDayGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__NHIndexTickHistOneDayGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'ticker', u'tradeDate', u'tradeTime', u'updateSequence', u'clearingDay', u'volume', u'value', u'lastPrice', u'highPrice', u'lowPrice', u'openPrice', u'preClosePrice', u'openInt']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'ticker': 'str','tradeTime': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __NHFutureIndexTickRTSnapshotGet(ticker = "", field = "", pandas = "1"):
    """
    高频数据，获取南华指数最新市场信息快照，包含以下六个指数：南华工业品指数、南华能化指数、南华贵金属指数、南华商品指数、南华金属指数、南华农产品指数
    
    :param ticker: NHCI-南华工业品指数、NHCI-南华能化指数、NHCI-南华贵金属指数、NHCI-南华商品指数、NHCI-南华金属指数、NHCI-南华农产品指数,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/idxMarket/getNHFutureIndexTickRTSnapshot.csv?ispandas=1&') 
    requestString.append("ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__NHFutureIndexTickRTSnapshotGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__NHFutureIndexTickRTSnapshotGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'clearingDay', u'dataDate', u'dataTime', u'highPrice', u'lastPrice', u'lowPrice', u'openInterest', u'openPrice', u'prevClosePrice', u'ticker', u'value', u'volume']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'clearingDay': 'str','dataDate': 'str','dataTime': 'str','ticker': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __NHIndexTickRTIntraDayGet(ticker, field = "", pandas = "1"):
    """
    高频数据,获取一只南华期货指数在当日内的Tick数据,包含以下六个指数：南华工业品指数、南华能化指数、南华贵金属指数、南华商品指数、南华金属指数、南华农产品指数
    
    :param ticker: NHCI-南华工业品指数、NHCI-南华能化指数、NHCI-南华贵金属指数、NHCI-南华商品指数、NHCI-南华金属指数、NHCI-南华农产品指数
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/idxMarket/getNHIndexTickRTIntraDay.csv?ispandas=1&') 
    if not isinstance(ticker, str) and not isinstance(ticker, unicode):
        ticker = str(ticker)

    requestString.append("ticker=%s"%(ticker))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__NHIndexTickRTIntraDayGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__NHIndexTickRTIntraDayGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'clearingDay', u'dataDate', u'dataSeq', u'dataTime', u'highPrice', u'lastPrice', u'lowPrice', u'openInterest', u'openPrice', u'prevClosePrice', u'ticker', u'value', u'volume']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'dataTime': 'str','ticker': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __NHIndexBarRTIntraDayGet(ticker, field = "", pandas = "1"):
    """
    高频数据，获取一只南华指数当日分钟线数据，指数范围仅包含以下6个：南华工业品指数、南华能化指数、南华贵金属指数、南华商品指数、南华金属指数、南华农产品指数
    
    :param ticker: NHCI-南华工业品指数、NHCI-南华能化指数、NHCI-南华贵金属指数、NHCI-南华商品指数、NHCI-南华金属指数、NHCI-南华农产品指数
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/idxMarket/getNHIndexBarRTIntraDay.csv?ispandas=1&') 
    if not isinstance(ticker, str) and not isinstance(ticker, unicode):
        ticker = str(ticker)

    requestString.append("ticker=%s"%(ticker))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__NHIndexBarRTIntraDayGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__NHIndexBarRTIntraDayGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'dataDate', u'clearingDay', u'barTime', u'offSet', u'ticker', u'openPrice', u'highPrice', u'lowPrice', u'closePirce', u'volume', u'value', u'openInt']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'barTime': 'str','ticker': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __NHIndexBarHistOneDayGet(ticker, tradeDate, unit = "", field = "", pandas = "1"):
    """
    获取一只南华指数在历史某一日分钟线数据
    
    :param ticker: NHCI-南华工业品指数、NHCI-南华能化指数、NHCI-南华贵金属指数、NHCI-南华商品指数、NHCI-南华金属指数、NHCI-南华农产品指数
    :param tradeDate: 交易日期
    :param unit: 当前bar时间,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/idxMarket/getNHIndexBarHistOneDay.csv?ispandas=1&') 
    if not isinstance(ticker, str) and not isinstance(ticker, unicode):
        ticker = str(ticker)

    requestString.append("ticker=%s"%(ticker))
    if not isinstance(tradeDate, str) and not isinstance(tradeDate, unicode):
        tradeDate = str(tradeDate)

    requestString.append("&tradeDate=%s"%(tradeDate))
    if not isinstance(unit, str) and not isinstance(unit, unicode):
        unit = str(unit)

    requestString.append("&unit=%s"%(unit))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__NHIndexBarHistOneDayGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__NHIndexBarHistOneDayGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'tradeDate', u'ticker', u'secOffSet', u'barTime', u'closePrice', u'openPrice', u'highPrice', u'lowPrice', u'volume', u'value', u'openInt']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'tradeDate': 'str','ticker': 'str','barTime': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()




# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

import json

import requests
import os

from .config import *
from .version import __version__ as v

version = 'version=sdk_%s'%v


def format_header_param(name, value):
    value = '%s=%s' % (name, value)
    return value


class Data(object):

    def __init__(self, session, data_path):
        self.session = session
        
        if data_path:
            self.data_path = data_path
        else:
            self.data_path = os.getcwd()

        self.data = self.list()


    def backup(self):
        print("该方法已废弃，请使用 client.notebook.download(download_all=True) 下载所有的data文件。")


    def upload(self, filepath, target_path=''):

        self.data = self.list()

        if target_path.endswith('/'):
            target_path = target_path[:-1]

        if target_path.find('/')>=0:
            return '目标路径只支持一级文件夹，请检查/字符。'

        if target_path and target_path not in self.directories:
            self._mk_dir(target_path)

        filepath = os.path.join(self.data_path, filepath)

        try:
            file_info = os.stat(filepath)
            if file_info.st_size>=100*1024*1024:
                print('{}文件超过100兆，暂不支持上传。'.format(filepath))
                return

            f = open(filepath, 'rb')
        except:
            print ("无法打开文件 {}，可能是权限不足或者文件不存在".format(filepath))
            return

        self._upload_data(filepath, target_path)


    def _upload_data(self, filepath, target_path):

        old_format_param = requests.packages.urllib3.fields.format_header_param
        requests.packages.urllib3.fields.format_header_param = format_header_param

        if target_path:
            url = '%s/%s'%(MERCURY_URL, target_path)
        else:
            url = MERCURY_URL

        try:
            files = {'datafile': open(filepath, 'rb')}
            values = {}
            r = self.session.post('%s?%s'%(url, version), files=files, data=values).json()
            if 'name' in r:
                print ('数据文件 {} 上传成功'.format(filepath))
            else:
                print ('数据文件 {} 上传失败'.format(filepath))
                print (r)
        except:
            traceback.print_exc()
        finally:
            requests.packages.urllib3.fields.format_header_param = old_format_param
        

    def list_data(self):
        url = MERCURY_URL
        res = self.session.get('%s?recursion'%(url))
        if not res.ok:
            print ('请求超时，请重试，重试未成功请直接联系客服.')
            return 0

        data = res.json()
        if isinstance(data, dict) and data.get('code'):
            print(data)
            return []

        all_data = []
        self.directories = []

        for elem in data:
            name = elem.get('name')
            type = elem.get('type')

            if type == 'databook':
                all_data.append(name)

            elif type == 'directory':
                self.directories.append(name)

                sub_elems = elem.get('children', [])
                for sub_elem in sub_elems:
                    elem_name = sub_elem.get('name')
                    type = sub_elem.get('type')
                    if type == 'databook':
                        all_data.append('%s/%s'%(name, elem_name))

        return all_data


    def list(self):
        self.data = self.list_data()
        return self.data


    def download(self, filename='', target_path='', download_all=False):

        self.data = self.list()
        directories = self.directories

        if download_all:
            for i in self.data:
                self._download_file(i, target_path)

        elif type(filename) == list:
            for i in filename:
                if i in self.data:
                    self._download_file(i, target_path)
                elif i in directories:
                    self._download_dir(dir=i, target_path=target_path)
                else:
                    print ('文件不存在 {}'.format(i))

        elif isinstance(filename, (str, unicode)):
            if filename in self.data:
                self._download_file(filename, target_path)
            elif filename in directories:
                    self._download_dir(dir=filename, target_path=target_path)
            else:
                print ('文件不存在 {}'.format(filename))

        else:
            pass


    def _download_dir(self, dir, target_path):
        dir =dir.replace('/', '')
        dir_elems = [e for e in self.data if e.startswith('%s/'%dir)]
        for dir_elem in dir_elems:
            self._download_file(dir_elem, target_path)


    def _download_file(self, filename, target_path):
        url = DATA_URL

        dataurl = url + '/' + filename

        print('下载data：\n开始下载 ', filename)

        if target_path:
            data_path = target_path
        else:
            data_path = self.data_path

        file_path = os.path.join(data_path, filename)

        try:
            path, filename = file_path.rsplit('/', 1)
        except:
            path = ''

        if path:
            if not os.path.exists(path):
                os.mkdir(path)
                print('文件夹 %s 不存在，已自动创建。'%path)
            elif not os.path.isdir(path):
                print('%s非文件夹，请检查文件或修改路径。'%path)
                return

        with open(file_path, 'wb') as f:
            size_info = self.session.get('%s?%s&size=1'%(dataurl, version)).json()
            if size_info['size'] > size_info['today_volume']:
                print ('本功能限制每天下载的数据文件总量为100兆，' \
                      '您今天已经下载%d兆，该文件大小超出限制无法下载。' % (100 - size_info['today_volume']/1024/1024))
                return 0
            response = self.session.get('%s?%s'%(dataurl, version), stream=True)

            if not response.ok:
                if response.status_code == 413:
                    print('{}文件超过100兆，暂不支持下载。'.format(filename))
                else:
                    print ('下载文件 {} 过程中出错'.format(filename))
                return 0
            
            for chunk in response.iter_content(1024 * 100):
                print("...", end="")
                f.write(chunk)

        print('')

        print('完成下载 %s, 路径 %s' %(filename, file_path))


    def delete(self, filename):

        self.data = self.list()
        directories = self.directories

        if isinstance(filename, list):
            for f in filename:
                self._delete_file(f)
        elif isinstance(filename, (str, unicode)):
            if filename in directories:
                self._delete_dir(filename)
            else:
                self._delete_file(filename)


    def _delete_dir(self, dir):
        dir =dir.replace('/', '')
        dir_elems = [e for e in self.data if e.startswith('%s/'%dir)]
        for dir_elem in dir_elems:
            self._delete_file(dir_elem)


    def _delete_file(self, filename):
        url = MERCURY_URL
        dataurl = url + '/' + filename

        try:
            response = self.session.delete('%s?%s'%(dataurl, version))
            status_code = response.status_code
            if status_code == 204:
                print ('{}数据文件删除成功'.format(filename))
            elif status_code == 404:
                print('{}数据文件不存在'.format(filename))
            else:
                print (response.text)
        except:
            print ('删除过程出错：')
            traceback.print_exc()


    def _mk_dir(self, dir_name):
        url = MERCURY_URL
        dataurl = '%s?dir=%s' %(url, dir_name)

        try:
            self.session.post('%s?%s'%(dataurl, version))
            print('文件夹 %s不存在，已自动创建。'%dir_name)
        except:
            print ('创建文件夹出错：')
            traceback.print_exc()
        





















# -*- coding: UTF-8 -*-
from . import api_base
try:
    from StringIO import StringIO
except:
    from io import StringIO
import pandas as pd
import sys
from datetime import datetime
from .api_base import get_cache_key, get_data_from_cache, put_data_in_cache, pretty_traceback
import inspect
try:
    unicode
except:
    unicode = str

__doc__="朝阳永续"
def __ReportGGGet(secID = "", ticker = "", writeDate = "", orgName = "", BeginWriteDate = "", EndwriteDate = "", title = "", author = "", field = "", pandas = "1"):
    """
    朝阳永续数据库的研报基本信息以及预测信息，历史数据从2011年开始，覆盖A股及港股信息，内容包含公司EPS、PE、营业收入、净资产收益率、EV_EBITDA、净利润、评级以及目标价等预测信息。
    
    :param secID: 研究对象在通联内部的代码，可输入多个,可以是列表,secID、ticker、writeDate至少选择一个
    :param ticker: 研究对象上市代码，可输入多个,可以是列表,secID、ticker、writeDate至少选择一个
    :param writeDate: 书写日期,输入格式“YYYYMMDD”,secID、ticker、writeDate至少选择一个
    :param orgName: 研究机构名称，通过输入机构名称，获得该机构发布的研究报告,可以是列表,可空
    :param BeginWriteDate: 研报书写日期，书写起始日期，获得自该日期开始撰写的研究报告，输入格式“YYYYMMDD”,可空
    :param EndwriteDate: 研报书写日期，可获得到某日期之前撰写的研报数据，输入格式“YYYYMMDD”,可空
    :param title: 研报标题，可输入个别关键词，通过模糊匹配，获得相关研报,可空
    :param author: 研报作者名称，可输入多个,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/report/getReportGG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        writeDate = writeDate.strftime('%Y%m%d')
    except:
        writeDate = writeDate.replace('-', '')
    requestString.append("&writeDate=%s"%(writeDate))
    requestString.append("&orgName=")
    if hasattr(orgName,'__iter__') and not isinstance(orgName, str):
        if len(orgName) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = orgName
            requestString.append(None)
        else:
            requestString.append(','.join(orgName))
    else:
        requestString.append(orgName)
    try:
        BeginWriteDate = BeginWriteDate.strftime('%Y%m%d')
    except:
        BeginWriteDate = BeginWriteDate.replace('-', '')
    requestString.append("&BeginWriteDate=%s"%(BeginWriteDate))
    try:
        EndwriteDate = EndwriteDate.strftime('%Y%m%d')
    except:
        EndwriteDate = EndwriteDate.replace('-', '')
    requestString.append("&EndwriteDate=%s"%(EndwriteDate))
    if not isinstance(title, str) and not isinstance(title, unicode):
        title = str(title)

    requestString.append("&title=%s"%(title))
    requestString.append("&author=")
    if hasattr(author,'__iter__') and not isinstance(author, str):
        if len(author) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = author
            requestString.append(None)
        else:
            requestString.append(','.join(author))
    else:
        requestString.append(author)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__ReportGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__ReportGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'orgName', u'writeDate', u'secID', u'ticker', u'SType', u'secShortName', u'title', u'author', u'ForecastPeriodTime', u'ADJEPS', u'PE', u'Sales', u'PnetProfit', u'ROE', u'OprProfit', u'EV_EBITDA', u'Recommadation', u'ADJRecommadation', u'targetPrice', u'Content']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'orgName': 'str','secID': 'str','ticker': 'str','SType': 'str','secShortName': 'str','title': 'str','author': 'str','Recommadation': 'str','ADJRecommadation': 'str','Content': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __ReportadjustGGGet(ticker = "", RRID = "", currentCreateDate = "", beginDate = "", endDate = "", field = "", pandas = "1"):
    """
    朝阳永续对研报预测净利润、EPS等信息的调整数据
    
    :param ticker: 证券代码,可以是列表,ticker、RRID、currentCreateDate至少选择一个
    :param RRID: 研报ID,可以是列表,ticker、RRID、currentCreateDate至少选择一个
    :param currentCreateDate: 本次预测日期,输入格式“YYYYMMDD”,ticker、RRID、currentCreateDate至少选择一个
    :param beginDate: 查询开始日期，输入格式“YYYYMMDD”,可空
    :param endDate: 查询结束日期，输入格式“YYYYMMDD”,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/report/getReportadjustGG.csv?ispandas=1&') 
    requestString.append("ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    requestString.append("&RRID=")
    if hasattr(RRID,'__iter__') and not isinstance(RRID, str):
        if len(RRID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = RRID
            requestString.append(None)
        else:
            requestString.append(','.join(RRID))
    else:
        requestString.append(RRID)
    try:
        currentCreateDate = currentCreateDate.strftime('%Y%m%d')
    except:
        currentCreateDate = currentCreateDate.replace('-', '')
    requestString.append("&currentCreateDate=%s"%(currentCreateDate))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__ReportadjustGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__ReportadjustGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'RRID', u'ticker', u'orgName', u'forecastYear', u'currentCreateDate', u'previousCreateDate', u'currentForecastProfit', u'previousForecastProfit', u'currentForecastEPS', u'previousForecastEPS', u'profitAdjustFlag', u'intoDate', u'profitAdjustRate', u'EPSAdjustRate']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'RRID': 'str','ticker': 'str','orgName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __DerReportGGGet(secID = "", ticker = "", createDate = "", RRID = "", beginDate = "", endDate = "", field = "", pandas = "1"):
    """
    朝阳永续研报基础信息表，包含研报标题、摘要、研究员、研究机构以及评级等信息。
    
    :param secID: 研究对象在通联内部的代码，可输入多个,可以是列表,secID、ticker、createDate至少选择一个
    :param ticker: 证券代码,可以是列表,secID、ticker、createDate至少选择一个
    :param createDate: 报告撰写日期,输入格式“YYYYMMDD”,secID、ticker、createDate至少选择一个
    :param RRID: 研报ID,可以是列表,可空
    :param beginDate: 查询报告开始日期，输入格式“YYYYMMDD”,可空
    :param endDate: 查询报告结束日期，输入格式“YYYYMMDD”,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/report/getDerReportGG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        createDate = createDate.strftime('%Y%m%d')
    except:
        createDate = createDate.replace('-', '')
    requestString.append("&createDate=%s"%(createDate))
    requestString.append("&RRID=")
    if hasattr(RRID,'__iter__') and not isinstance(RRID, str):
        if len(RRID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = RRID
            requestString.append(None)
        else:
            requestString.append(','.join(RRID))
    else:
        requestString.append(RRID)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__DerReportGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__DerReportGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'RRID', u'secID', u'ticker', u'secShortName', u'title', u'content', u'typeID', u'type', u'orgID', u'orgName', u'author', u'scoreCD', u'score', u'organScoreCD', u'orgScore', u'createDate', u'intoDate', u'text1', u'text2', u'targetprice', u'text8', u'priceCurrent', u'capitalCurrent', u'attention', u'attentionName']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','ticker': 'str','secShortName': 'str','title': 'str','content': 'str','type': 'str','orgName': 'str','author': 'str','score': 'str','orgScore': 'str','text1': 'str','text2': 'str','attentionName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __cmbReportGGGet(secID = "", ticker = "", createDate = "", beginDate = "", endDate = "", field = "", pandas = "1"):
    """
    可获得朝阳永续研报的基本信息和预测信息,覆盖A股和B股。
    
    :param secID: 研究对象在通联内部的代码，可输入多个,可以是列表,secID、ticker、createDate至少选择一个
    :param ticker: 研究对象上市代码，可输入多个,可以是列表,secID、ticker、createDate至少选择一个
    :param createDate: 报告撰写日期,secID、ticker、createDate至少选择一个
    :param beginDate: 报告撰写日期，获得该日期之后撰写的数据，输入格式“YYYYMMDD”,可空
    :param endDate: 报告撰写日期，获得该日期之前撰写的数据，输入格式“YYYYMMDD”,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/report/getcmbReportGG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        createDate = createDate.strftime('%Y%m%d')
    except:
        createDate = createDate.replace('-', '')
    requestString.append("&createDate=%s"%(createDate))
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__cmbReportGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__cmbReportGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'rrID', u'secID', u'originID', u'ticker', u'secShortName', u'Title', u'content', u'typeID', u'partyID', u'author', u'scoreCD', u'organScoreCD', u'createDate', u'intoDate', u'Text3', u'Text5', u'Text8', u'priceCurrent', u'capitalCurrent', u'Attention', u'attentionName', u'scoreFlag', u'updateTime']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','originID': 'str','ticker': 'str','secShortName': 'str','Title': 'str','content': 'str','author': 'str','Text3': 'str','attentionName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __cmbReportSubGGGet(rrID = "", field = "", pandas = "1"):
    """
    获得对应朝阳永续预测表、报告预测数据合并表的预测数据，包含预期营业收入、预期每股收益、预期净利润等预测数据。
    
    :param rrID: 对应朝阳永续预测表、报告预测数据合并表中的rrID,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/report/getcmbReportSubGG.csv?ispandas=1&') 
    requestString.append("rrID=")
    if hasattr(rrID,'__iter__') and not isinstance(rrID, str):
        if len(rrID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = rrID
            requestString.append(None)
        else:
            requestString.append(','.join(rrID))
    else:
        requestString.append(rrID)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__cmbReportSubGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__cmbReportSubGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'rrID', u'originID', u'year', u'quarter', u'foreIncome', u'foreProfit', u'foreIncomeShare', u'foreReturnCashShare', u'foreReturnCapitalShare', u'foreReturn', u'rTar3', u'rTar5', u'profitFlag', u'updateTime']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __derReportResGGGet(secID = "", ticker = "", beginDate = "", endDate = "", field = "", pandas = "1"):
    """
    朝阳永续研报基础信息表，包含研报标题、摘要、研究员、研究机构以及评级等信息。
    
    :param secID: 研究对象在通联内部的代码，可输入多个,可以是列表,secID、ticker至少选择一个
    :param ticker: 研究对象上市代码，可输入多个,可以是列表,secID、ticker至少选择一个
    :param beginDate: 报告撰写日期，获得该日期之后撰写的数据，输入格式“YYYYMMDD”,可空
    :param endDate: 报告撰写日期，获得该日期之前撰写的数据，输入格式“YYYYMMDD”,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/report/getderReportResGG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__derReportResGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__derReportResGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'rrID', u'secID', u'ticker', u'secShortName', u'title', u'content', u'typeId', u'partyID', u'author', u'scoreCD', u'organScoreCD', u'createTime', u'intoTime', u'text5', u'text8', u'priceCurrent', u'capitalCurrent', u'attention', u'attentionName', u'scoreFlag', u'updateTime']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','ticker': 'str','secShortName': 'str','title': 'str','content': 'str','author': 'str','attentionName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __derReportSubGGGet(rrID = "", field = "", pandas = "1"):
    """
    获得对应朝阳永续研究报告（个股）的预测数据，包含预期营业收入、预期每股收益、预期净利润等预测数据。
    
    :param rrID: 对应朝阳永续研究报告信息（个股）表中的rrID,可以是列表,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/report/getderReportSubGG.csv?ispandas=1&') 
    requestString.append("rrID=")
    if hasattr(rrID,'__iter__') and not isinstance(rrID, str):
        if len(rrID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = rrID
            requestString.append(None)
        else:
            requestString.append(','.join(rrID))
    else:
        requestString.append(rrID)
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__derReportSubGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__derReportSubGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'rrID', u'year', u'quarter', u'foreIncome', u'foreProfit', u'foreIncomeShare', u'foreReturnCashShare', u'foreReturnCapitalShare', u'foreReturn', u'rTar3', u'rTar5', u'profitFlag', u'updateTime']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __CGRDReportGGGet(secID = "", ticker = "", BeginPubDate = "", EndPubDate = "", field = "", pandas = "1"):
    """
    朝阳永续研报一致性预期数据表的评级数据，从2011年开始，内容涵盖A股市场。
    
    :param secID: 研究对象在通联内部的代码，可输入多个,可以是列表,secID、ticker至少选择一个
    :param ticker: 研究对象上市代码，可输入多个,可以是列表,secID、ticker至少选择一个
    :param BeginPubDate: 数据发布日期，获得该日期之后发布的数据，输入格式“YYYYMMDD”,可空
    :param EndPubDate: 数据发布日期，获得该日期之前发布的数据，输入格式“YYYYMMDD”,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getCGRDReportGG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        BeginPubDate = BeginPubDate.strftime('%Y%m%d')
    except:
        BeginPubDate = BeginPubDate.replace('-', '')
    requestString.append("&BeginPubDate=%s"%(BeginPubDate))
    try:
        EndPubDate = EndPubDate.strftime('%Y%m%d')
    except:
        EndPubDate = EndPubDate.replace('-', '')
    requestString.append("&EndPubDate=%s"%(EndPubDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__CGRDReportGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__CGRDReportGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'secID', u'SType', u'ticker', u'publishDate', u'secShortName', u'targetPrice', u'targetPriceType', u'Rating', u'RatingType']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','SType': 'str','ticker': 'str','secShortName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __CESTReportGGGet(secID = "", ticker = "", BeginPubDate = "", EndPubDate = "", field = "", pandas = "1"):
    """
    朝阳永续研报一致性预期数据表中的预测数据，从2011年开始，涵盖A股市场，内容包括EPS、PE、营业收入、净利润等。
    
    :param secID: 研究对象在通联内部的代码，可输入多个,可以是列表,secID、ticker至少选择一个
    :param ticker: 研究对象上市代码，可输入多个,可以是列表,secID、ticker至少选择一个
    :param BeginPubDate: 数据发布日期，获得该日期之后发布的数据，输入格式“YYYYMMDD”,可空
    :param EndPubDate: 数据发布日期，获得该日期之前发布的数据，输入格式“YYYYMMDD”,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getCESTReportGG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        BeginPubDate = BeginPubDate.strftime('%Y%m%d')
    except:
        BeginPubDate = BeginPubDate.replace('-', '')
    requestString.append("&BeginPubDate=%s"%(BeginPubDate))
    try:
        EndPubDate = EndPubDate.strftime('%Y%m%d')
    except:
        EndPubDate = EndPubDate.replace('-', '')
    requestString.append("&EndPubDate=%s"%(EndPubDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__CESTReportGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__CESTReportGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'secID', u'SType', u'ticker', u'publishDate', u'secShortName', u'ForecastPeriodTime', u'EPS_con', u'EPSType', u'Pnetprofit_con', u'PnetprofitType', u'PE_con', u'PEG_con', u'ROE_con', u'STOCK_TYPE', u'CON_HISDATE', u'C4_HISDATE', u'C3', u'C7', u'CB', u'PB_CON', u'INCOME', u'INCOME_TYPE']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','SType': 'str','ticker': 'str','secShortName': 'str','INCOME_TYPE': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __conIncomeGGGet(ticker, beginDate = "", endDate = "", field = "", pandas = "1"):
    """
    朝阳永续对各证券的一致性预期收入的数据信息
    
    :param ticker: 证券代码,可以是列表
    :param beginDate: 查询开始日期,可空
    :param endDate: 查询结束日期,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getconIncomeGG.csv?ispandas=1&') 
    requestString.append("ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    if not isinstance(beginDate, str) and not isinstance(beginDate, unicode):
        beginDate = str(beginDate)

    requestString.append("&beginDate=%s"%(beginDate))
    if not isinstance(endDate, str) and not isinstance(endDate, unicode):
        endDate = str(endDate)

    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__conIncomeGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__conIncomeGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'ticker', u'tdate', u'rptDate', u'incomeType', u'income', u'incomeHisdate']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'ticker': 'str','incomeType': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __CESTStockGGGet(secID = "", ticker = "", BeginPubDate = "", EndPubDate = "", field = "", pandas = "1"):
    """
    可获得朝阳永续个股一致预期数据，包含一致预期EPS 、一致预期PE等
    
    :param secID: 研究对象在通联内部的代码，可输入多个,可以是列表,secID、ticker至少选择一个
    :param ticker: 研究对象上市代码，可输入多个,可以是列表,secID、ticker至少选择一个
    :param BeginPubDate: 数据发布日期，获得该日期之后发布的数据，输入格式“YYYYMMDD”,可空
    :param EndPubDate: 数据发布日期，获得该日期之前发布的数据，输入格式“YYYYMMDD”,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getCESTStockGG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        BeginPubDate = BeginPubDate.strftime('%Y%m%d')
    except:
        BeginPubDate = BeginPubDate.replace('-', '')
    requestString.append("&BeginPubDate=%s"%(BeginPubDate))
    try:
        EndPubDate = EndPubDate.strftime('%Y%m%d')
    except:
        EndPubDate = EndPubDate.replace('-', '')
    requestString.append("&EndPubDate=%s"%(EndPubDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__CESTStockGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__CESTStockGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'secID', u'SType', u'ticker', u'secShortName', u'stockType', u'publishDate', u'rptDate', u'rptType', u'EPSCon', u'EPSType', u'conHisdate', u'pnetprofitCon', u'pnetprofitType', u'C4Hisdate', u'PECon', u'PEGCon', u'ROECon', u'C3', u'C7', u'C80', u'C81', u'C82', u'C83', u'C84', u'CB', u'PBCon', u'updateTime']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','SType': 'str','ticker': 'str','secShortName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __SWconForeGGGet(ticker = "", industry = "", beginDate = "", endDate = "", tmStamp = "", field = "", pandas = "1"):
    """
    获得朝阳永续一致预期（申万行业）数据，包括一致预期EPS，一致预期PE,一致预期ROE，一致预期PB等数据。
    
    :param ticker: 行业代码,可以是列表,ticker、industry至少选择一个
    :param industry: 行业名称,可以是列表,ticker、industry至少选择一个
    :param beginDate: 数据预测日期，获得该日期之后发布的数据，输入格式“YYYYMMDD”,可空
    :param endDate: 数据预测日期，获得该日期之后发布的数据，输入格式“YYYYMMDD”,可空
    :param tmStamp: 通过判断下载标识的变动便于及时发现数据的增量及更新。,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getSWconForeGG.csv?ispandas=1&') 
    requestString.append("ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    requestString.append("&industry=")
    if hasattr(industry,'__iter__') and not isinstance(industry, str):
        if len(industry) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = industry
            requestString.append(None)
        else:
            requestString.append(','.join(industry))
    else:
        requestString.append(industry)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    if not isinstance(tmStamp, str) and not isinstance(tmStamp, unicode):
        tmStamp = str(tmStamp)

    requestString.append("&tmStamp=%s"%(tmStamp))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__SWconForeGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__SWconForeGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'ticker', u'industry', u'tickerType', u'conDate', u'rptDate', u'rptType', u'C1', u'conType', u'C4', u'c4Type', u'C3', u'C5', u'C6', u'C7', u'C12', u'CB', u'CPB', u'tmStamp']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'ticker': 'str','industry': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __SWconForeC2GGGet(ticker = "", industry = "", beginDate = "", endDate = "", tmStamp = "", field = "", pandas = "1"):
    """
    获得朝阳永续一致预期衍生指标计算(申万行业)数据，包括滚动EPS，滚动净利润，滚动PE等数据。
    
    :param ticker: 行业代码,可以是列表,ticker、industry至少选择一个
    :param industry: 行业名称,可以是列表,ticker、industry至少选择一个
    :param beginDate: 数据预测日期，获得该日期之后发布的数据，输入格式“YYYYMMDD”,可空
    :param endDate: 数据预测日期，获得该日期之后发布的数据，输入格式“YYYYMMDD”,可空
    :param tmStamp: 通过判断下载标识的变动便于及时发现数据的增量及更新。,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getSWconForeC2GG.csv?ispandas=1&') 
    requestString.append("ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    requestString.append("&industry=")
    if hasattr(industry,'__iter__') and not isinstance(industry, str):
        if len(industry) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = industry
            requestString.append(None)
        else:
            requestString.append(','.join(industry))
    else:
        requestString.append(industry)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    if not isinstance(tmStamp, str) and not isinstance(tmStamp, unicode):
        tmStamp = str(tmStamp)

    requestString.append("&tmStamp=%s"%(tmStamp))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__SWconForeC2GGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__SWconForeC2GGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'ticker', u'industry', u'tickerType', u'conDate', u'C13', u'C9', u'tmStamp']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'ticker': 'str','industry': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __SWconForeC3GGGet(ticker = "", industry = "", beginDate = "", endDate = "", tmStamp = "", field = "", pandas = "1"):
    """
    获得朝阳永续一致预期滚动衍生数据表(申万行业)数据，包括滚动净资产，滚动市净率（滚动PB），滚动净利润复合增长率等数据。
    
    :param ticker: 行业代码,可以是列表,ticker、industry至少选择一个
    :param industry: 行业名称,可以是列表,ticker、industry至少选择一个
    :param beginDate: 数据预测日期，获得该日期之后发布的数据，输入格式“YYYYMMDD”,可空
    :param endDate: 数据预测日期，获得该日期之后发布的数据，输入格式“YYYYMMDD”,可空
    :param tmStamp: 通过判断下载标识的变动便于及时发现数据的增量及更新。,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getSWconForeC3GG.csv?ispandas=1&') 
    requestString.append("ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    requestString.append("&industry=")
    if hasattr(industry,'__iter__') and not isinstance(industry, str):
        if len(industry) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = industry
            requestString.append(None)
        else:
            requestString.append(','.join(industry))
    else:
        requestString.append(industry)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    if not isinstance(tmStamp, str) and not isinstance(tmStamp, unicode):
        tmStamp = str(tmStamp)

    requestString.append("&tmStamp=%s"%(tmStamp))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__SWconForeC3GGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__SWconForeC3GGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'ticker', u'industry', u'tickerType', u'conDate', u'CGB', u'CGPB', u'CGG', u'CGPEG', u'tmStamp']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'ticker': 'str','industry': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __conForeIdxC2GGGet(secID = "", ticker = "", beginDate = "", endDate = "", tmStamp = "", field = "", pandas = "1"):
    """
    获得朝阳一致预期衍生指标计算(指数)，包括滚动净利润，滚动PE等数据。
    
    :param secID: 证券内部编码，可通过交易代码和交易市场在DataAPI.SecIDGet获取到。,可以是列表,secID、ticker至少选择一个
    :param ticker: 交易代码,可以是列表,secID、ticker至少选择一个
    :param beginDate: 预测日期（开始日期）,可空
    :param endDate: 预测日期（截止日期）,可空
    :param tmStamp: 通过判断下载标识的变动便于及时发现数据的增量及更新。,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getconForeIdxC2GG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    if not isinstance(tmStamp, str) and not isinstance(tmStamp, unicode):
        tmStamp = str(tmStamp)

    requestString.append("&tmStamp=%s"%(tmStamp))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__conForeIdxC2GGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__conForeIdxC2GGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'secID', u'ticker', u'secShortName', u'tickerType', u'conDate', u'C13', u'C9', u'tmStamp']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','ticker': 'str','secShortName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __conForeStkC2GGGet(secID = "", ticker = "", beginDate = "", endDate = "", tmStamp = "", field = "", pandas = "1"):
    """
    获得朝阳永续一致预期衍生指标计算(个股)，包括滚动EPS，滚动净利润,滚动PE等数据。
    
    :param secID: 证券内部编码，可通过交易代码和交易市场在DataAPI.SecIDGet获取到。,可以是列表,secID、ticker至少选择一个
    :param ticker: 交易代码,可以是列表,secID、ticker至少选择一个
    :param beginDate: 预测日期（开始日期）,可空
    :param endDate: 预测日期（截止日期）,可空
    :param tmStamp: 通过判断下载标识的变动便于及时发现数据的增量及更新。,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getconForeStkC2GG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    if not isinstance(tmStamp, str) and not isinstance(tmStamp, unicode):
        tmStamp = str(tmStamp)

    requestString.append("&tmStamp=%s"%(tmStamp))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__conForeStkC2GGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__conForeStkC2GGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'secID', u'ticker', u'secShortName', u'conDate', u'C2', u'C13', u'C9', u'tmStamp']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','ticker': 'str','secShortName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __conForeIdxC3GGGet(secID = "", ticker = "", beginDate = "", endDate = "", tmStamp = "", field = "", pandas = "1"):
    """
    获得朝阳永续一致预期滚动衍生数据(指数)，包括滚动净资产，滚动市净率（滚动PB）,滚动净利润复合增长率，滚动PEG等数据。
    
    :param secID: 证券内部编码，可通过交易代码和交易市场在DataAPI.SecIDGet获取到。,可以是列表,secID、ticker至少选择一个
    :param ticker: 交易代码,可以是列表,secID、ticker至少选择一个
    :param beginDate: 预测日期（开始日期）,可空
    :param endDate: 预测日期（截止日期）,可空
    :param tmStamp: 通过判断下载标识的变动便于及时发现数据的增量及更新。,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getconForeIdxC3GG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    if not isinstance(tmStamp, str) and not isinstance(tmStamp, unicode):
        tmStamp = str(tmStamp)

    requestString.append("&tmStamp=%s"%(tmStamp))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__conForeIdxC3GGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__conForeIdxC3GGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'secID', u'ticker', u'stockName', u'stockType', u'conDate', u'CGB', u'CGPB', u'CGG', u'CGPEG', u'tmStamp']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','ticker': 'str','stockName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __conForeStkC3GGGet(secID = "", ticker = "", beginDate = "", endDate = "", tmStamp = "", field = "", pandas = "1"):
    """
    获得朝阳永续一致预期滚动衍生数据(个股)，包括滚动净资产，滚动市净率（滚动PB）,滚动净利润复合增长率，滚动PEG等数据。
    
    :param secID: 证券内部编码，可通过交易代码和交易市场在DataAPI.SecIDGet获取到。,可以是列表,secID、ticker至少选择一个
    :param ticker: 交易代码,可以是列表,secID、ticker至少选择一个
    :param beginDate: 预测日期（开始日期）,可空
    :param endDate: 预测日期（截止日期）,可空
    :param tmStamp: 通过判断下载标识的变动便于及时发现数据的增量及更新。,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getconForeStkC3GG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    if not isinstance(tmStamp, str) and not isinstance(tmStamp, unicode):
        tmStamp = str(tmStamp)

    requestString.append("&tmStamp=%s"%(tmStamp))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__conForeStkC3GGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__conForeStkC3GGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'secID', u'ticker', u'secShortName', u'conDate', u'CGB', u'CGPB', u'CGG', u'CGPEG', u'tmStamp']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','ticker': 'str','secShortName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __conForeIdxGGGet(secID = "", ticker = "", beginDate = "", endDate = "", tmStamp = "", field = "", pandas = "1"):
    """
    获得朝阳永续一致预期（指数）数据，包括一致预期EPS，一致预期PE,一致预期ROE，一致预期PB等数据。
    
    :param secID: 证券内部编码，可通过交易代码和交易市场在DataAPI.SecIDGet获取到。,可以是列表,secID、ticker至少选择一个
    :param ticker: 交易代码,可以是列表,secID、ticker至少选择一个
    :param beginDate: 预测日期（开始日期）,可空
    :param endDate: 预测日期（截止日期）,可空
    :param tmStamp: 通过判断下载标识的变动便于及时发现数据的增量及更新。,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/consensus/getconForeIdxGG.csv?ispandas=1&') 
    requestString.append("secID=")
    if hasattr(secID,'__iter__') and not isinstance(secID, str):
        if len(secID) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = secID
            requestString.append(None)
        else:
            requestString.append(','.join(secID))
    else:
        requestString.append(secID)
    requestString.append("&ticker=")
    if hasattr(ticker,'__iter__') and not isinstance(ticker, str):
        if len(ticker) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = ticker
            requestString.append(None)
        else:
            requestString.append(','.join(ticker))
    else:
        requestString.append(ticker)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    if not isinstance(tmStamp, str) and not isinstance(tmStamp, unicode):
        tmStamp = str(tmStamp)

    requestString.append("&tmStamp=%s"%(tmStamp))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__conForeIdxGGGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__conForeIdxGGGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'secID', u'ticker', u'secShortName', u'conDate', u'c4Type', u'rptDate', u'rptType', u'C1', u'C3', u'C4', u'C5', u'C6', u'C7', u'C12', u'CB', u'CPB', u'tmStamp']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'secID': 'str','ticker': 'str','secShortName': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()




# -*- coding: UTF-8 -*-
from . import api_base
try:
    from StringIO import StringIO
except:
    from io import StringIO
import pandas as pd
import sys
from datetime import datetime
from .api_base import get_cache_key, get_data_from_cache, put_data_in_cache, pretty_traceback
import inspect
try:
    unicode
except:
    unicode = str

__doc__="中国债券信息网"
def __YieldCurveGet(CurveName, Tenor, CurveType = "", BeginDate = "", EndDate = "", field = "", pandas = "1"):
    """
    输入中债收益率曲线名称、曲线类型、待偿期限，可查询到中债在这些条件下的收益率
    
    :param CurveName: 收益率曲线名称，输入如中债收益率曲线名称“中国固定利率国债收益率曲线”,可同时输入多个收益率曲线名称,可以是列表
    :param Tenor: 待偿期限，以年为单位，如1个月，输入"0.08",,可同时输入多个待偿期限,可以是列表
    :param CurveType: 收益率曲线类型，可选：到期、即期、远期的即期、远期的到期，默认为"到期",可空
    :param BeginDate: 数据起始日期，默认今天，输入格式“YYYYMMDD”,可空
    :param EndDate: 数据起始日期，默认今天，输入格式“YYYYMMDD”,可空
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/bond/getYieldCurve.csv?ispandas=1&') 
    requestString.append("CurveName=")
    if hasattr(CurveName,'__iter__') and not isinstance(CurveName, str):
        if len(CurveName) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = CurveName
            requestString.append(None)
        else:
            requestString.append(','.join(CurveName))
    else:
        requestString.append(CurveName)
    requestString.append("&Tenor=")
    if hasattr(Tenor,'__iter__') and not isinstance(Tenor, str):
        if len(Tenor) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = Tenor
            requestString.append(None)
        else:
            requestString.append(','.join(Tenor))
    else:
        requestString.append(Tenor)
    if not isinstance(CurveType, str) and not isinstance(CurveType, unicode):
        CurveType = str(CurveType)

    requestString.append("&CurveType=%s"%(CurveType))
    try:
        BeginDate = BeginDate.strftime('%Y%m%d')
    except:
        BeginDate = BeginDate.replace('-', '')
    requestString.append("&BeginDate=%s"%(BeginDate))
    try:
        EndDate = EndDate.strftime('%Y%m%d')
    except:
        EndDate = EndDate.replace('-', '')
    requestString.append("&EndDate=%s"%(EndDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__YieldCurveGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__YieldCurveGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'CurveName', u'CurveType', u'Tenor', u'DataDate', u'Yield']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'CurveName': 'str','CurveType': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __CBBenchmarkIndicesGet(benchmarkName = "", valuationType = "", beginDate = "", endDate = "", field = "", pandas = "1"):
    """
    获取中债指数各代偿期财富指数、净价指数、全价指数、平均市值法久期、平均市值法凸性等信息
    
    :param benchmarkName: 中债指数指数名称,可以是列表,benchmarkName、valuationType、beginDate、endDate至少选择一个
    :param valuationType: 如以下：10年以上；1-3年；1年以下；3-5年；5-7年；7-10年；总值,可以是列表,benchmarkName、valuationType、beginDate、endDate至少选择一个
    :param beginDate: 查询起始时间，格式为YYYYMMDD,benchmarkName、valuationType、beginDate、endDate至少选择一个
    :param endDate: 查询截止时间，格式为YYYYMMDD,benchmarkName、valuationType、beginDate、endDate至少选择一个
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/idxMarket/getCBBenchmarkIndices.csv?ispandas=1&') 
    requestString.append("benchmarkName=")
    if hasattr(benchmarkName,'__iter__') and not isinstance(benchmarkName, str):
        if len(benchmarkName) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = benchmarkName
            requestString.append(None)
        else:
            requestString.append(','.join(benchmarkName))
    else:
        requestString.append(benchmarkName)
    requestString.append("&valuationType=")
    if hasattr(valuationType,'__iter__') and not isinstance(valuationType, str):
        if len(valuationType) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = valuationType
            requestString.append(None)
        else:
            requestString.append(','.join(valuationType))
    else:
        requestString.append(valuationType)
    try:
        beginDate = beginDate.strftime('%Y%m%d')
    except:
        beginDate = beginDate.replace('-', '')
    requestString.append("&beginDate=%s"%(beginDate))
    try:
        endDate = endDate.strftime('%Y%m%d')
    except:
        endDate = endDate.replace('-', '')
    requestString.append("&endDate=%s"%(endDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__CBBenchmarkIndicesGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__CBBenchmarkIndicesGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'dataDate', u'benchmarkName', u'valuationTypeCD', u'valuationType', u'weightedAvgMktCap', u'weightedAvgGrossPx', u'weightedAvgNetPx', u'mktValDura', u'cashflowDiscDura', u'mktValConv', u'cashflowDiscConv', u'yieldToMaturity', u'baseRate', u'yearsToMaturity', u'weightedAvgCoupon', u'indexValue', u'mktCapIncrement', u'grossPxIncrement', u'netPxIncrement', u'mktValYtm', u'cashbondNav']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'benchmarkName': 'str','valuationTypeCD': 'str','valuationType': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()

def __CBBenchmarkCompositionGet(benchmarkName = "", valuationTypeDesc = "", dataDate = "", field = "", pandas = "1"):
    """
    获取中债指数样本券代码、样本券交易市场、样本券净价、全价、到期收益率、指数市值权重等信息
    
    :param benchmarkName: 中债指数指数名称,benchmarkName、valuationTypeDesc、dataDate至少选择一个
    :param valuationTypeDesc: 如以下：10年以上；1-3年；1年以下；3-5年；5-7年；7-10年；总值,benchmarkName、valuationTypeDesc、dataDate至少选择一个
    :param dataDate: 日期，格式为YYYYMMDD,benchmarkName、valuationTypeDesc、dataDate至少选择一个
    :param field: 所需字段,可以是列表,可空
    :param pandas: 1表示返回 pandas data frame，0表示返回csv,可空
    :return: :raise e: API查询的结果，是CSV或者被转成pandas data frame；若查询API失败，返回空data frame； 若解析失败，则抛出异常
    """
        
    pretty_traceback()
    frame = inspect.currentframe()
    func_name, cache_key = get_cache_key(frame)
    cache_result = get_data_from_cache(func_name, cache_key)
    if cache_result is not None:
        return cache_result
    split_index = None
    split_param = None
    httpClient = api_base.__getConn__()    
    requestString = []
    requestString.append('/api/idxWeight/getCBBenchmarkComposition.csv?ispandas=1&') 
    if not isinstance(benchmarkName, str) and not isinstance(benchmarkName, unicode):
        benchmarkName = str(benchmarkName)

    requestString.append("benchmarkName=%s"%(benchmarkName))
    if not isinstance(valuationTypeDesc, str) and not isinstance(valuationTypeDesc, unicode):
        valuationTypeDesc = str(valuationTypeDesc)

    requestString.append("&valuationTypeDesc=%s"%(valuationTypeDesc))
    try:
        dataDate = dataDate.strftime('%Y%m%d')
    except:
        dataDate = dataDate.replace('-', '')
    requestString.append("&dataDate=%s"%(dataDate))
    requestString.append("&field=")
    if hasattr(field,'__iter__') and not isinstance(field, str):
        if len(field) > 100 and split_param is None:
            split_index = len(requestString)
            split_param = field
            requestString.append(None)
        else:
            requestString.append(','.join(field))
    else:
        requestString.append(field)
    if split_param is None:
        csvString = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
        if csvString is None or len(csvString) == 0 or (csvString[0] == '-' and not api_base.is_no_data_warn(csvString, False)) or csvString[0] == '{':
            api_base.handle_error(csvString, '__CBBenchmarkCompositionGet')
        elif csvString[:2] == '-1':
            csvString = ''
    else:
        p_list = api_base.splist(split_param, 100)
        csvString = []
        for index, item in enumerate(p_list):
            requestString[split_index] = ','.join(item)
            temp_result = api_base.__getCSV__(''.join(requestString), httpClient, gw=True)
            if temp_result is None or len(temp_result) == 0 or temp_result[0] == '{' or (temp_result[0] == '-' and not api_base.is_no_data_warn(temp_result, False)):
                api_base.handle_error(temp_result, '__CBBenchmarkCompositionGet')
            if temp_result[:2] != '-1':
                csvString.append(temp_result if len(csvString) == 0 else temp_result[temp_result.find('\n')+1:])
        csvString = ''.join(csvString)
    
    if len(csvString) == 0:
        if 'field' not in locals() or len(field) == 0:
            field = [u'dataDate', u'benchmarkName', u'valuationTypeCD', u'valuationTypeDesc', u'xibeTicker', u'bondShortName', u'marketCD', u'market', u'grossPx', u'netPx', u'accruedInterest', u'increment', u'yearsToMaturity', u'yieldToMaturity', u'spread', u'modifiedDuration', u'spotDuration', u'spreadDuratoin', u'convexity', u'spotConvexity', u'spreadConvexity', u'baseRate', u'indexWeight']
        if hasattr(field, '__iter__') and not isinstance(field, str):
            csvString = ','.join(field) + '\n'
        else:
            csvString = field + '\n'
    if pandas != "1":
        put_data_in_cache(func_name, cache_key, csvString)
        return csvString
    try:
        myIO = StringIO(csvString)
        pdFrame = pd.read_csv(myIO, dtype = {'benchmarkName': 'str','valuationTypeCD': 'str','valuationTypeDesc': 'str','xibeTicker': 'str','bondShortName': 'str','marketCD': 'str','market': 'str'},  )
        put_data_in_cache(func_name, cache_key, pdFrame)
        return pdFrame
    except Exception as e:
        raise e
    finally:
        myIO.close()




# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals


import sys
from . import uqer
from .uqer import Client
from .mfclient import neutralize, standardize, winsorize, simple_long_only, long_only
from . import DataAPI

from .version import __version__

from .DataAPI import retry_interval, max_retries


try:
    DataAPI.api_base.replace_api_files()
    for e in list(sys.modules.keys()):
        if e.startswith('DataAPI') or e.startswith('uqer.DataAPI'):
            del sys.modules[e]
    from . import DataAPI
except:
    import traceback
    print(traceback.format_exc())
    print('upgrade fail.')


# -*- coding: utf-8 -*-

import json
import requests
import pandas as pd
import datetime as dt

from .config import *
from .utils import convert_date

class LiveTrading(object):
    """优矿模拟交易
    """
    def __init__(self, session):
        super(LiveTrading, self).__init__()
        self.session = session

    def list(self):
        """获取当前模拟交易策略列表
        """
        r = self.session.get(Live_Trading_URL)
        df = pd.DataFrame.from_records(r.json())
        if not df.empty:
            df = df[['id', 'name', 'start_time', 'daily_return', 'total_return']]
            df.start_time = df.start_time.apply(lambda x: dt.datetime.fromtimestamp(x / 1000))

        print ('当前模拟交易个数：{}'.format(len(df)))
        print (df)

        return df

    def orders(self, strategy_id, trade_date=None):
        """单个模拟交易策略详情
            strategy_id: 策略 id
            trade_date: 交易日，如果未指定，则返回当日调仓情况
        """

        url = Live_Order_URL.format(strategy_id)
        if trade_date:
            url = '{}?date={}'.format(url, convert_date(trade_date))
        else:
            url = '{}?calender=1'.format(url)

        try:
            r = self.session.get(url)
            df = pd.DataFrame.from_records(r.json())
            return df
        except:
            print ('获取模拟交易调仓信号错误或者当前没有模拟交易')







            
    


# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

import requests
import os


from uqer.config import Get_Factor, Upload_FactorData, Create_Factor
from uqer.factor import load_df_from_file, check_factor_data


class UserFactor(object):
    """优矿用户因子
    """
    def __init__(self, session):
        super(UserFactor, self).__init__()
        self.session = session

    def _get_uploaded_factor(self):
        """获取上传的因子列表

        """
        r, data = get_factors(self.session)
        if not r:
            print (data)
            return []
        return map(lambda item: {'factor_name': item['factor_name'], 'chinese_name': item['chinese_name'],
                                 'category': item['category'], 'begin_tradedate': str(item['begin_tradedate']),
                                 'end_tradedate': str(item['end_tradedate']), 'pool_name': item['pool_name']},
                   filter(lambda item: True if 'upload_factor' in item and item['upload_factor'] else False, data))

    def get_private_uploaded_factor(self):
        result = self._get_uploaded_factor()
        return filter(lambda item: True if item['pool_name'] == 'private' else False, result)

    def get_factor_name(self, f_id):
        r, data = get_factor(self.session, f_id)
        if not r:
            return None
        return data['factor_name']

    def submit_private_factor(self, factor_name, chinese_name, chinese_description, factor_data_file_path,
                       function_list="winsorize,neutralize,standardize", universe="A"):
        """
        创建一个新的私有因子，并上传因子数据，触发因子报告计算
        :param factor_name: 因子名
        :param chinese_name: 因子中文简称
        :param chinese_description: 因子中文描述
        :param category: 因子分类，用逗号分开多个分类，候选值是：Value,Quality,Momentum,Growth,Sentiment,Return_Risk,Derive,Analyst_FORECAST,Common_Index,Per_share_Indicators,Others
        :param factor_data_file_path: 因子数据文件
        :param function_list: 信号处理函数，用逗号分开多个函数，候选值是：winsorize,neutralize,standardize
        :param universe: 投资域选择，用逗号分开多个投资域，候选值是：A, HS300, ZZ500
        :return: (code, data), code可选范围200,400，403,490,409,500。其中code为200时，data是创建的因子id。code为其他时，data是错误信息提示，
                            其中，400表示有输入错误；403表示没有权限；490表示众筹因子达到一天数量限制；409表示因子名字有冲突；500表示服务器内部错误
        """
        return self._submit_factor(factor_name, chinese_name, chinese_description, '', factor_data_file_path,
                                   function_list, universe, factor_type='private')

    def _submit_factor(self, factor_name, chinese_name, chinese_description, category, factor_data_file_path,
                       function_list="winsorize,neutralize,standardize", universe="A", factor_type='public'):
        """
        创建一个新的因子，并上传因子数据，触发因子报告计算
        :param factor_name: 因子名
        :param chinese_name: 因子中文简称
        :param chinese_description: 因子中文描述
        :param category: 因子分类，用逗号分开多个分类，候选值是：Value,Quality,Momentum,Growth,Sentiment,Return_Risk,Derive,Analyst_FORECAST,Common_Index,Per_share_Indicators,Others;
                        空字符串表示没有分类
        :param factor_data_file_path: 因子数据文件
        :param function_list: 信号处理函数，用逗号分开多个函数，候选值是：winsorize,neutralize,standardize
        :param universe: 投资域选择，用逗号分开多个投资域，候选值是：A, HS300, ZZ500
        :param factor_type: public表示创建到海量因子池，private表示创建到私有因子
        :return: (code, data), code可选范围200,400，403,490,409,500。其中code为200时，data是创建的因子id。code为其他时，data是错误信息提示，
                            其中，400表示有输入错误；403表示没有权限；490表示众筹因子达到一天数量限制；409表示因子名字有冲突；500表示服务器内部错误
        """
        for function in function_list.split(','):
            if function not in ("winsorize", "neutralize", "standardize"):
                msg = '信号处理方法错误：%s' % function
                print (msg)
                return 400, msg
        for item in universe.split(','):
           if item not in ('A', 'HSSLL', 'ZZWLL', 'HS300', 'ZZ500'):
               msg = '投资域选项必须是A，HS300，ZZ500'
               print (msg)
               return 400, msg
        for item in category.split(','):
            if item not in (
            'Value', 'Quality', 'Momentum', 'Growth', 'Sentiment', 'Return_Risk', 'Derive', 'Analyst_FORECAST',
            'Common_Index', 'Per_share_Indicators', 'Others', ''):
                msg = '因子分类错误：%s' % item
                print (msg)
                return 400, msg
        if factor_type not in ('public', 'private'):
            msg = 'factor_type %s 不合理' % factor_type
            print (msg)
            return 400, msg
        print ('正在加载因子csv数据...')
        df, msg = load_df_from_file(factor_data_file_path)
        if df is None:
            print (msg)
            return 400, msg
        print ('加载因子csv数据完成')
        print ('正在校验因子csv数据格式...')
        r, msg = check_factor_data(df)
        if not r:
            print (msg)
            return 400, msg
        print ('校验因子csv数据格式完成')
        print ('正在提交因子...')
        r, f_id = create_factor(self.session, factor_name, category, chinese_name, chinese_description,
                                      function_list.split(','), universe, factor_type)
        if r != 200:
            msg = '创建因子失败：%s' % f_id
            print (msg)
            return r, msg
        print ('提交因子完成\n正在上传因子数据并触发因子报告计算...')
        r, msg = upload_factor_data(self.session, f_id, factor_data_file_path)
        if not r:
            msg = '上传因子数据失败%s，因子id:%s，因子名%s，您后续可以使用更新因子数据方法重新上传因子数据并触发因子报告计算' % (msg, f_id, factor_name)
            print(msg)
            return 500, msg
        import time
        time.sleep(3)
        count = 0
        succ = False
        while True:
            if check_factor_upload_done(self.session, f_id):
                succ = True
                break
            if count >= 50:
                break
            time.sleep(10)
            count += 1
        real_factor_name = self.get_factor_name(f_id)
        if succ:
            print ('上传因子数据完成')
        else:
            print ('上传因子数据遇到问题，可能是服务器正忙，请通过相关API查看上传数据是否到位，或者重新操作')
            msg = '创建因子成功，上传因子数据失败，因子id:%s, 因子名:%s' % (f_id, real_factor_name)
            print (msg)
            return 500, msg
        print ('创建因子成功，因子id:%s, 因子名:%s' %(f_id, real_factor_name))
        return 200, f_id

    def update_private_factor_data(self, factor_name, factor_data_file_path):
        """
        更新私有因子数据，并触发因子报告计算
        :param factor_name: 因子名
        :param factor_data_file_path: 因子数据文件地址
        :return:(code, data), code可选范围200,400，404,500。其中code为200时，data是创建的因子id。code为其他时，data是错误信息提示，
                            其中，400表示有输入错误；404表示因子不存在；500表示服务器内部错误
        """
        return self._update_factor_data(factor_name, factor_data_file_path, factor_type='private')

    def _update_factor_data(self, factor_name, factor_data_file_path, factor_type='public'):
        """
        更新因子数据，并触发因子报告计算
        :param factor_name: 因子名
        :param factor_data_file_path: 因子数据文件地址
        :param factor_type: public表示海量因子，private表示私有因子
        :return: (code, data), code可选范围200,400，404,500。其中code为200时，data是创建的因子id。code为其他时，data是错误信息提示，
                            其中，400表示有输入错误；404表示因子不存在；500表示服务器内部错误
        """
        print ('正在加载因子csv数据...')
        df, msg = load_df_from_file(factor_data_file_path)
        if df is None:
            print (msg)
            return 400,
        print ('加载因子csv数据完成')
        print ('正在校验因子csv数据格式...')
        r, msg = check_factor_data(df)
        if not r:
            print (msg)
            return 400, msg
        print ('校验因子csv数据格式完成\n正在上传因子数据...')
        r, msg = update_uploaded_factor_data(self.session, factor_name, factor_data_file_path, factor_type)
        if not r:
            msg = '上传因子数据失败%s，因子名%s，您后续可以使用更新因子数据方法重新上传因子数据并触发因子报告计算' % (msg, factor_name)
            print (msg)
            return 500, msg
        f_id = msg
        import time
        time.sleep(10)
        count = 0
        succ = False
        while True:
            if check_factor_upload_done(self.session, f_id):
                succ = True
                break
            if count >= 60:
                break
            time.sleep(10)
            count += 1
        if succ:
            print ('上传因子数据完成')
        else:
            msg = '上传因子数据遇到问题，可能是服务器正忙，请通过相关API查看上传数据是否到位，或者重新操作\n因子id:%s, 因子名:%s' % (f_id, factor_name)
            print (msg)
            return 500, msg
        print ('更新因子数据成功，因子id: %s，因子名:%s' % (f_id, factor_name))
        return 200, f_id

def create_factor(session, factor_name, 
	              category, 
	              chinese_name, 
	              chinese_des,
                  function_list=["winsorize", "neutralize", "standardize"], 
                  universe='A',
                  factor_type='public'):
    url = Create_Factor
    input_json = {
                    'pool_name': factor_type,
                    'factor_name': factor_name, 
                    'code': '', 
                    'category': category,
                    'chinese_des': chinese_des, 
                    'upload_factor': True, 
                    'chinese_name': chinese_name,
                    'function_list': function_list, 
                    'universe': universe
                }
    try:
        r = session.post(url, json=input_json)
        status_code, info = r.status_code, r.text
        return status_code, info
    except:
        import traceback
        return 500, '调用上传因子服务失败，%s' % traceback.format_exc()


def get_factors(session):
    url = Get_Factor
    import json
    try:
        r = session.get(url + '?need_max_min_td=1')
        status_code, info = r.status_code, r.text
        if status_code == 200:
            return True, json.loads(info)
        return False, info
    except:
        import traceback
        return False, '获取因子失败，%s' % traceback.format_exc()

def get_factor(session, f_id):
    url = Get_Factor
    import json
    try:
        r = session.get('%s/%s' % (url, f_id))
        status_code, info = r.status_code, r.text
        if status_code == 200:
            return True, json.loads(info)
        return False, info
    except:
        import traceback
        return False, '获取因子失败，%s' % traceback.format_exc()


def upload_factor_data(session, f_id, file_path):
    import zlib
    with open(file_path) as f:
        data = f.read()
        try:
            data = zlib.compress(data)
        except:
            data = zlib.compress(bytes(data, encoding='utf-8'))
        url = Upload_FactorData
        try:
            r = session.post(url + '?f_id=%s' % f_id, data=data)
            status_code, msg = r.status_code, r.text
            if status_code == 200:
                return True, msg
            return False, msg
        except:
            import traceback
            return False, '调用因子数据上传服务失败，%s' % traceback.format_exc()


def check_factor_upload_done(session, f_id):
    try:
        url = Upload_FactorData
        r = session.get(url + '?f_id=%s' % f_id, timeout=30)
        status_code, msg = r.status_code, r.text
        if status_code == 200:
            if msg == 'Done':
                return True
    except:
        pass
    return False


def update_uploaded_factor_data(session, factor_name, file_path, factor_type='public'):
    import zlib
    with open(file_path) as f:
        data = f.read()
        try:
            data = zlib.compress(data)
        except:
            data = zlib.compress(bytes(data, encoding='utf-8'))
        url = Upload_FactorData
        try:
            r = session.put(url + '?f_id=%s&trigger_gen_report=%d&pool_name=%s' % (factor_name, 0, factor_type), data=data)
            status_code, msg = r.status_code, r.text
            if status_code == 200:
                return True, msg
            return False, msg
        except:
            import traceback
            return False, '调用因子数据更新服务失败，%s' % traceback.format_exc()

